;+
; NAME:
;   Main.pro 
;
; PURPOSE:
;
; This program reads in the colocated AIRS and Cloudsat fields, and plots maps and vertical x-sections. This code can
; be adapted to other coincident platforms, such as MODIS, etc.
;
; AUTHOR:
;
;   Brian H. Kahn
;   Jet Propulsion Laboratory
;   4800 Oak Grove Drive
;   MS 169-237
;   Pasadean, CA  91109
;   Tel. 818-393-0676
;
; KNOWN ISSUES:
;
;   * No functionality (yet) to read Cloudsat granules thatpan across two contiguous days (one every ~16 Cloudsat granules)
;   * For the AIRS granules approaching the poles, there is a plotting problem for the x-sections, such that only the
;     data up to the highest (positive or negative) latitude is plotted.  Will be fixed in future. This does not affect the
;     matching of the data, only the plotting.
;   * The reflectivity range is hardwired in, in some sense.  The colorscale does not necessarily match the actual values (for now).
;   * Paths to the Cloudsat and AIRS data are hardwired in.
;   * This code is provided as is.  As the author obtains time, he will continue to modify and improve this routine
;
; EXAMPLE:
;   
;  @Compile
;  main, 2006, 7, 20, 1216, 1  
;        [year], [month], [day], [cloudsat granule], [plotting switch]
;  To continue one AIRS granule at a time, follow with  ".c"
;
;  The "1" at the end indicates you wish to plot the horizontal fields and vertical x-sections. "0" will switch the plotting off.
;
; MODIFICATION HISTORY:
;
;   Version 1.0, written August 31st, 2006
;

PRO  Main, year, month, day, cs_granule, nplot_switch

  year = string(year,format='(i4.4)')
  month = string(month,format='(i2.2)')
  day = string(day,format='(i2.2)')
  cs_granule = string(cs_granule,format='(i5.5)')

  infile_cloudsat = year+'?????????_'+cs_granule+'_CS_2B-GEOPROF_GRANULE_P_R??_E??.hdf'
  collocate_file = 'lat_lon_match_'+month+'_'+day+'_'+year+'_???_???_'+cs_granule+'.dat'

; Cloudsat L2 cloud boundary granule (1 orbit), Jay Mace owns this product
  pattern = '/home/kahn/Cloudsat/'+infile_cloudsat   

; Read Cloudsat granule
  read_Cloudsat_geoprof, pattern, Profile_time, UTC_start, TAI_start, Latitude, Longitude, Height, $
                         Spacecraft_altitude, DEM_elevation, RayHeader_RangeBinSize, Data_quality, Data_status,  $
                         Data_targetID, SurfaceHeightBin, CPR_Cloud_mask, Gaseous_Attenuation, Radar_Reflectivity, $
                         MODIS_Cloud_Fraction, MODIS_scene_char, MODIS_scene_var, CPR_Echo_Top, sem_NoiseFloor

  max_Cloudsat = n_elements(Latitude)-1

  CPR_Cloud_MASK_2 = dblarr(max_Cloudsat,125)
  Radar_Reflectivity_2 = dblarr(max_Cloudsat,125)
  Height_2 = dblarr(max_Cloudsat,125)

  for i = 0, 124  do begin
     for j = 0.0, max_Cloudsat-1.0  do begin
        CPR_Cloud_MASK_2[j,i] = CPR_Cloud_MASK[i,j]
        Height_2[j,i] = Height[i,j]
        Radar_Reflectivity_2[j,i] = Radar_Reflectivity[i,j]
     endfor
  endfor

; Open up and read AIRS/Cloudsat matched lat/lon pairs

  header = ''
  fp_list = dblarr(max_Cloudsat)
  scan_list = dblarr(max_Cloudsat)
  granule_list = dblarr(max_Cloudsat)
  cloudsat_lat = dblarr(max_Cloudsat)
  cloudsat_lon = dblarr(max_Cloudsat)
  airs_lat = dblarr(max_Cloudsat)
  airs_lon = dblarr(max_Cloudsat)
  PCldTopUpper = dblarr(max_Cloudsat)
  PCldTopLower = dblarr(max_Cloudsat)
  ZCldTopUpper = dblarr(max_Cloudsat)
  ZCldTopLower = dblarr(max_Cloudsat)
  CldFrcUpper = fltarr(max_Cloudsat)
  CldFrcLower = fltarr(max_Cloudsat)
  CldFrcUpper2 = fltarr(max_Cloudsat)
  CldFrcLower2 = fltarr(max_Cloudsat)
  nCldFrcFlag = dblarr(max_Cloudsat)
  
  nCldFrcFlag[*] = 0 
  ii = 1.0

  if size(month,/type) ne 7 then  month = string(month,format='(i2.2)')
  if size(day,/type) ne 7 then   day = string(day,format='(i2.2)')
  if size(year,/type) ne 7 then   year = string(year,format='(i4.4)')

  openr, 1, collocate_file 
  for i = 0.0, max_Cloudsat-1.0  do begin
     readf, 1, header    
     column_entries = separate_elements(header)
     fp_list[i] = string_to_real(column_entries(0))
     scan_list[i] = string_to_real(column_entries(1))
     granule_list[i] = string_to_real(column_entries(2))
     cloudsat_lat[i] = string_to_real(column_entries(3))
     cloudsat_lon[i] = string_to_real(column_entries(5))
     airs_lat[i] = string_to_real(column_entries(4))
     airs_lon[i] = string_to_real(column_entries(6))
     PCldTopUpper[i] = string_to_real(column_entries(7))
     PCldTopLower[i] = string_to_real(column_entries(8))
     ZCldTopUpper[i] = string_to_real(column_entries(9))
     ZCldTopLower[i] = string_to_real(column_entries(10))
  endfor
  close, 1

; Loop through AIRS granules
  for a = min(granule_list), max(granule_list), 1.0D  do begin
;  for a = 23, 23, 1.0D  do begin

     granule = string(a,format='(i3.3)')

     pattern2 = '/home/kahn/Cloudsat/AIRS/' + $
;     pattern2 = '/archive/AIRSOps/airs/gdaac/c3_v4/'+year+'/'+month+'/'+day+'/airx2ret/' + $
                'AIRS.'+year+'.'+month+'.'+day+'.'+granule+'.L2.RetStd.v4.*.????????????.hdf'

     read_swath_l2, pattern2, numfp2, numline2, tai2, lat, lon, pres, tair, h2o, retqa=retqa, TSurface=TSurface, $
                    TSurfErr=TSurfErr, CldFrc=CldFrc, PCldTop=PCldTop, TCldTop=TCldTop, TotH2OStd=TotH2OStd, totH2OErr=totH2OErr, $
                    CloudFrcErr=CloudFrcErr, PCloudTopErr=PCloudTopErr, TCloudTopErr=TCloudTopErr, TSAir=TSAir, GP_Height=GP_Height

     PCldTopUp = dblarr(numfp2,numline2)
     PCldTopLow = dblarr(numfp2,numline2)
     CldFrcUp = dblarr(numfp2*3,numline2*3)
     CldFrcLow = dblarr(numfp2*3,numline2*3)
     for k = 0, numfp2-1 do begin
        for l = 0, numline2-1 do begin
           PCldTopUp[k,l] = PCldTop[0,k,l] 
           PCldTopLow[k,l] = PCldTop[1,k,l] 
           for m = 0, 2  do begin
              for n = 0, 2  do begin
                 CldFrcUp[k*3+m,l*3+n] = CldFrc[0,m,n,k,l] 
                 CldFrcLow[k*3+m,l*3+n] = CldFrc[1,m,n,k,l]
              endfor
           endfor
        endfor
     endfor

     match = where(granule_list eq a)

     for b = min(match), max(match)  do begin
          for m = 0, 2  do begin
               for n = 0, 2  do begin
                    CldFrcUpper[b] = CldFrcUpper[b] + CldFrcUp[m+fp_list[b]*3.0,n+scan_list[b]*3.0]
                    CldFrcLower[b] = CldFrcLower[b] + CldFrcLow[m+fp_list[b]*3.0,n+scan_list[b]*3.0]
               endfor
          endfor
     endfor
     CldFrcUpper = CldFrcUpper/9.0
     CldFrcLower = CldFrcLower/9.0

     for b = min(match), max(match)  do begin
        CldFrcUpper2[b] = CldFrcUp[1.0+fp_list[b]*3.0,1.0+scan_list[b]*3.0]
        CldFrcLower2[b] = CldFrcLow[1.0+fp_list[b]*3.0,1.0+scan_list[b]*3.0]
     endfor

;     print, PCldTopUpper[min(match):max(match)], CldFrcUpper[min(match):max(match)], PCldTopLower[min(match):max(match)], CldFrcLower[min(match):max(match)]

     if(nplot_switch eq 1) then begin
          Device, Decomposed=0
          !p.background = 0
          loadct, file='colors1.tbl', 42
          @set_color_ae

          window, 0, retain=2, xsize=560, ysize=640
          lambrigtsen_map, lat, lon, PCldTopUp,  min=50., max=1000., /colorbar, format='(f6.1)'
          oplot, cloudsat_lon[match], cloudsat_lat[match], psym=3, color=white
     endif

     if(nplot_switch eq 1) then begin

          pattern_L1B = '/home/kahn/Cloudsat/AIRS/' + $
;          pattern_L1B = '/archive/AIRSOps/airs/gdaac/c3_v4/'+year+'/'+month+'/'+day+'/airibrad/' + $
          'AIRS.'+year+'.'+month+'.'+day+'.'+granule+'.L1B.AIRS_Rad.v4.0.*.????????????.hdf'

          read_swath_l1_airs, pattern_L1B, numfp, numline, tai, lat_L1B, lon_L1B, rad, solzen, freq=freq, scanang=scanang

          Tb_960_a = reform ( brtemp ( freq[901], rad[901,*,*] ), numfp, numline )
          Tb_960_b = reform ( brtemp ( freq[902], rad[902,*,*] ), numfp, numline )
          Tb_960 = (Tb_960_a + Tb_960_b)/2.0
   
     endif

     if(nplot_switch eq 1) then begin

          window, 1, retain=2, xsize=560, ysize=640
          lambrigtsen_map, lat_L1B, lon_L1B, Tb_960,  min=190., max=320., /colorbar, format='(f6.1)'
          oplot, cloudsat_lon[match], cloudsat_lat[match], psym=3, color=white

     endif

     if(nplot_switch eq 1) then begin

          Tb_1231 = reform ( brtemp ( freq[1290], rad[1290,*,*] ), numfp, numline )

          phase_btd = Tb_1231 - Tb_960

          Tb_2616 = reform ( brtemp ( freq[2332], rad[2332,*,*] ), numfp, numline )
   
          sw_lw_difference = Tb_2616 - Tb_960

     endif

     if(nplot_switch eq 1) then begin
          window, 2, retain=2, xsize=560, ysize=640
          lambrigtsen_map, lat_L1B, lon_L1B, phase_btd,  min=-5., max=5., /colorbar, format='(f6.1)'
          oplot, cloudsat_lon[match], cloudsat_lat[match], psym=3, color=white

          window, 3, retain=2, xsize=560, ysize=640
          lambrigtsen_map, lat_L1B, lon_L1B, sw_lw_difference,  min=-10., max=30., /colorbar, format='(f6.1)'
          oplot, cloudsat_lon[match], cloudsat_lat[match], psym=3, color=white

          window, 4, retain=2, xsize=560, ysize=640
          lambrigtsen_map, lat_L1B, lon_L1B, CldFrcUp,  min=0., max=1., /colorbar, format='(f6.2)'
          oplot, cloudsat_lon[match], cloudsat_lat[match], psym=3, color=white

          window, 5, retain=2, xsize=560, ysize=640
          lambrigtsen_map, lat_L1B, lon_L1B, CldFrcLow,  min=0., max=1., /colorbar, format='(f6.2)'
          oplot, cloudsat_lon[match], cloudsat_lat[match], psym=3, color=white
     endif

     for ll = 0, 124   do begin

        cpr_cloud_mask_2_temp = fltarr(n_elements(CPR_CLOUD_MASK_2[*,ll]))
        radar_reflectivity_2_temp = fltarr(n_elements(CPR_CLOUD_MASK_2[*,ll]))

        cpr_cloud_mask_2_temp = CPR_CLOUD_MASK_2[*,ll]
        radar_reflectivity_2_temp = Radar_Reflectivity_2[*,ll]

; Mask out all cloud mask boundaries greater than 40.0
        match_reflectivity = where( (cpr_cloud_mask_2_temp gt 40.0) )
        if(match_reflectivity[0] ne -1) then  cpr_cloud_mask_2_temp[match_reflectivity] = 0.0
        CPR_CLOUD_MASK_2[match,ll] = cpr_cloud_mask_2_temp[match]

; Mask out all cloud mask boundaries less than 10.0
        match_reflectivity = where( (cpr_cloud_mask_2_temp le 10.0) )
        if(match_reflectivity[0] ne -1) then  cpr_cloud_mask_2_temp[match_reflectivity] = 0.0
        CPR_CLOUD_MASK_2[match,ll] = cpr_cloud_mask_2_temp[match]

; Mask out all reflectivities greater than 60 dBz
        match_reflectivity = where( (cpr_cloud_mask_2_temp le 10.0) or (radar_reflectivity_2_temp ge 6000.0) )
;        match_reflectivity = where( cpr_cloud_mask_2_temp le 10.0 )
        if(match_reflectivity[0] ne -1) then  radar_reflectivity_2_temp[match_reflectivity] = -5100. 
        Radar_Reflectivity_2[match,ll] = radar_reflectivity_2_temp[match]

     endfor

     test = where( (MODIS_scene_char lt 2) or (MODIS_scene_char gt 5) ) 
     MODIS_scene_char[test] = -999 

     if(nplot_switch eq 1) then begin

          Device, Decomposed=0
          levels = 40
          LoadCT, 33, NColors=50, Bottom=3

          window, 6, retain=2, ysize=600, xsize=1200

          plot, Latitude[match], MODIS_Cloud_Fraction[match]/4., linestyle=0, color=white, xtitle=' Latitude ',  $
                ytitle=' Height (km) '

          contour, CPR_CLOUD_MASK_2[match,*], Latitude[match], Height_2[match,*]/1000., /Fill,  $
                   C_Colors=Indgen(levels)+3, background=255, nlevels=levels, position=[0.1,0.1,0.9,0.8], $
                   color=black, xrange=[Latitude[min(match)], Latitude[max(match)]], yrange=[0,20],  $
                   xtitle=' Latitude ', ytitle=' Height (km) '

          colorbar, ncolors = 50, bottom=3, divisions=5, range=[0,50], format='(i4)', position=[0.1,0.9,0.9,0.95], color=black,  $
                    title='CloudSat Reflectivity (dBZ)'

          oplot, Latitude[match], MODIS_scene_char[match]+25., psym=3, color=black
          oplot, Latitude[match], ZCldTopUpper[match]/1000., psym=3, color=black
          oplot, Latitude[match], ZCldTopLower[match]/1000., psym=3, color=black

; This sets up the circle
          radius = 1.
          nSeg = 30
          frac = 1.
          nFrac = FIX( frac * nSeg+1)
          cir = FINDGEN( nFrac ) * 2.*!PI / FLOAT( nSeg )
     endif

     temp1 = min(match)
     temp2 = max(match)

     if(nplot_switch eq 1) then begin

          for aa = temp1, temp2-1.0, 1.0  do begin
               if(aa eq temp1) then  nCldFrcFlag[aa] = ii
               if( (aa le max(match)-1) and ( (CldFrcLower[aa+1] ne CldFrcLower[aa]) or (CldFrcUpper[aa+1] ne CldFrcUpper[aa]) ) ) then begin
;                    print, CldFrcUpper[aa], aa, min(match), max(match), nCldFrcFlag[aa], ii
                    ii = ii + 1
               endif
               nCldFrcFlag[aa+1] = ii 
          endfor

          temp3 = nCldFrcFlag(temp1) 
          temp4 = nCldFrcFlag(temp2) 
          for bb = temp3, temp4, 1.0  do begin
               match_cldfrc = where(nCldFrcFlag eq bb)
               min_match_cldfrc = min(match_cldfrc)
               max_match_cldfrc = max(match_cldfrc)
               diff_match_cldfrc = max_match_cldfrc - min_match_cldfrc
               midpoint = fix(diff_match_cldfrc/2.)+min_match_cldfrc
               radius = CldFrcUpper[midpoint]*5
               radius2 = CldFrcLower[midpoint]*5
               if(radius lt 0.0) then  radius = 0.0
               if(radius2 lt 0.0) then  radius2 = 0.0
               USERSYM, [radius*COS(cir)], [radius*SIN(cir)]
               oplot, [Latitude[midpoint]], [ZCldTopUpper[midpoint]/1000.], psym=8, color=black
               USERSYM, [radius2*COS(cir)], [radius2*SIN(cir)]
               oplot, [Latitude[midpoint]], [ZCldTopLower[midpoint]/1000.], psym=8, color=black
          endfor

          temp3 = nCldFrcFlag(temp1)
          temp4 = nCldFrcFlag(temp2)
          for bb = temp3, temp4, 1.0  do begin
               match_cldfrc = where(nCldFrcFlag eq bb)
               min_match_cldfrc = min(match_cldfrc)
               max_match_cldfrc = max(match_cldfrc)
               diff_match_cldfrc = max_match_cldfrc - min_match_cldfrc
               midpoint = fix(diff_match_cldfrc/2.)+min_match_cldfrc
               radius = CldFrcUpper2[midpoint]*5
               radius2 = CldFrcLower2[midpoint]*5
               if(radius lt 0.0) then  radius = 0.0
               if(radius2 lt 0.0) then  radius2 = 0.0
               USERSYM, [radius*COS(cir)], [radius*SIN(cir)]
               oplot, [Latitude[midpoint]], [ZCldTopUpper[midpoint]/1000.], psym=8, color=4
               USERSYM, [radius2*COS(cir)], [radius2*SIN(cir)]
               oplot, [Latitude[midpoint]], [ZCldTopLower[midpoint]/1000.], psym=8, color=5
          endfor

          Device, Decomposed=0
          levels = 70
          LoadCT, 33, NColors=100, Bottom=3

          step = (max(Radar_Reflectivity_2[match,*]) - (-4000)) / levels
          user_levels = IndGen(levels) * step + (-4000)
          print, user_levels
          print, min(Radar_Reflectivity_2[match,*]), max(Radar_Reflectivity_2[match,*])

          window, 7, retain=2, ysize=600, xsize=1200

          plot, Latitude[match], MODIS_Cloud_Fraction[match]/4., linestyle=0, color=white, xtitle=' Latitude ',  $
                ytitle=' Height (km) '

          contour, Radar_Reflectivity_2[match,*], Latitude[match], Height_2[match,*]/1000., /Fill,  $
                   C_Colors=Indgen(levels)+3, background=255, levels=user_levels, position=[0.1,0.1,0.9,0.8], color=black,  $
                   xrange=[Latitude[min(match)], Latitude[max(match)]], yrange=[0,20], xtitle=' Latitude ', ytitle=' Height (km) '

          colorbar, ncolors = 100, bottom=3, divisions=8, range=[-4000,5000], format='(i7)', position=[0.1,0.9,0.9,0.95],  $
                    title='CloudSat Reflectivity (dBZ)', color=black
;          colorbar, ncolors = 256, bottom=3, divisions=9, range=[-4000,5000], format='(i7)', position=[0.1,0.9,0.9,0.95], color=black

          oplot, Latitude[match], ZCldTopUpper[match]/1000., psym=3, color=black
          oplot, Latitude[match], ZCldTopLower[match]/1000., psym=3, color=black

          temp3 = nCldFrcFlag(temp1)
          temp4 = nCldFrcFlag(temp2)
          for bb = temp3, temp4, 1.0  do begin
               match_cldfrc = where(nCldFrcFlag eq bb)
               min_match_cldfrc = min(match_cldfrc)
               max_match_cldfrc = max(match_cldfrc)
               diff_match_cldfrc = max_match_cldfrc - min_match_cldfrc
               midpoint = fix(diff_match_cldfrc/2.)+min_match_cldfrc
               radius = CldFrcUpper[midpoint]*5
               radius2 = CldFrcLower[midpoint]*5
               if(radius lt 0.0) then  radius = 0.0
               if(radius2 lt 0.0) then  radius2 = 0.0
               USERSYM, [radius*COS(cir)], [radius*SIN(cir)]
               oplot, [Latitude[midpoint]], [ZCldTopUpper[midpoint]/1000.], psym=8, color=black
               USERSYM, [radius2*COS(cir)], [radius2*SIN(cir)]
               oplot, [Latitude[midpoint]], [ZCldTopLower[midpoint]/1000.], psym=8, color=black
          endfor

     endif

     temp5 = strcompress(string(min(granule_list),format='(i3.3)'),/remove_all)
     temp6 = strcompress(string(max(granule_list),format='(i3.3)'),/remove_all)

     openw, 5, 'match_'+month+'_'+day+'_'+year+'_'+temp5+'_'+temp6+'_'+cs_granule+'.dat', /append
     for aa = temp1, temp2-1.0, 1.0  do begin
          highest_cloudsat_cloud_6 = where(CPR_CLOUD_MASK_2[aa,*] ge 6.0)
          highest_cloudsat_cloud_10 = where(CPR_CLOUD_MASK_2[aa,*] gt 10.0)
          highest_cloudsat_cloud_20 = where(CPR_CLOUD_MASK_2[aa,*] ge 20.0)
          if(highest_cloudsat_cloud_6[0] ne -1) then  cs_cloud_top_6 = max(Height_2[aa,highest_cloudsat_cloud_6])
          if(highest_cloudsat_cloud_6[0] eq -1) then  cs_cloud_top_6 = -9999.0
          if(highest_cloudsat_cloud_10[0] ne -1) then  cs_cloud_top_10 = max(Height_2[aa,highest_cloudsat_cloud_10])
          if(highest_cloudsat_cloud_10[0] eq -1) then  cs_cloud_top_10 = -9999.0
          if(highest_cloudsat_cloud_20[0] ne -1) then  cs_cloud_top_20 = max(Height_2[aa,highest_cloudsat_cloud_20])
          if(highest_cloudsat_cloud_20[0] eq -1) then  cs_cloud_top_20 = -9999.0
          print, fp_list[aa], scan_list[aa], granule_list[aa], cloudsat_lat[aa], airs_lat[aa], cloudsat_lon[aa], airs_lon[aa],  $
                 ZCldTopUpper[aa], ZCldTopLower[aa], cs_cloud_top_6, cs_cloud_top_10, cs_cloud_top_20, CldFrcUpper[aa], $
                 CldFrcLower[aa], CldFrcUpper2[aa], CldFrcLower2[aa], format='(1x,2i4,i5,4f9.3,5f9.1,4f8.4)' 
          printf, 5, fp_list[aa], scan_list[aa], granule_list[aa], cloudsat_lat[aa], airs_lat[aa], cloudsat_lon[aa], airs_lon[aa],  $
                 ZCldTopUpper[aa], ZCldTopLower[aa], cs_cloud_top_6, cs_cloud_top_10, cs_cloud_top_20, CldFrcUpper[aa], $
                 CldFrcLower[aa], CldFrcUpper2[aa], CldFrcLower2[aa], format='(1x,2i4,i5,4f9.3,5f9.1,4f8.4)' 
     endfor
     close, 5

;     wset, 4
;     x2tiff, 'tiff/airs_cloudsat_ucf_'+month+'_'+day+'_'+year+'_'+granule+'.tiff'
;     wset, 7
;     x2tiff, 'tiff/airs_cloudsat_reflectivity_xsect_'+month+'_'+day+'_'+year+'_'+granule+'.tiff'

     if(nplot_switch eq 1) then  stop 

  endfor

stop

end
